/**
 * calculator.js - JavaScript functions for Radio-Theory HF Antenna Calculator
 * Updated with proper scientific formula for antenna length calculation
 * And improved antenna ranking system
 */

// Wait for the DOM to be fully loaded before attaching event listeners
document.addEventListener('DOMContentLoaded', function() {
    console.log("DOM loaded - attaching event listeners");
    
    // Directly attach event listeners to the buttons using their IDs
    document.getElementById('calculateBtn').addEventListener('click', calculateAntennas);
    document.getElementById('clearBtn').addEventListener('click', clearInput);
    document.getElementById('printBtn').addEventListener('click', printResults);
    
    // Add event listener for the antenna rankings toggle button
    const toggleButton = document.getElementById('antennaRankingsToggle');
    if (toggleButton) {
        toggleButton.addEventListener('click', toggleAntennaRankings);
    }
});

// Define color palette for antenna groups - using more subtle colors
const colorPalette = [
    '#7fb17f', // Muted Green
    '#7fadd4', // Muted Blue
    '#a07979', // Muted Burgundy (replacing Red)
    '#a090c0', // Muted Purple
    '#c9b991', // Muted Gold (replacing Orange)
    '#9d8e84', // Muted Brown
    '#8a9ba7', // Muted Blue Gray
    '#92c2c9', // Muted Cyan
    '#b4ad7e', // Muted Olive
    '#adc49f'  // Muted Light Green
];

// Store calculation results globally for export functions
let lastCalculationResults = {
    channels: [],
    groups: []
};

/**
 * Toggle the visibility of antenna rankings section
 */
function toggleAntennaRankings() {
    const contentSection = document.getElementById('antennaRankingsContent');
    const iconElement = this.querySelector('.collapsible-icon');
    
    if (contentSection.style.display === 'none') {
        contentSection.style.display = 'block';
        iconElement.textContent = '▼';
    } else {
        contentSection.style.display = 'none';
        iconElement.textContent = '►';
    }
}

/**
 * Calculate antenna dimensions based on frequency and wire type using proper scientific formula
 * L_full = (983.571 × VF) / f_MHz
 */
function calculateAntennas() {
    console.log("Calculate function called");
    
    // Get wire type selection and set proper velocity factor (VF)
    const w16 = document.getElementById("WF16");
    const w1 = document.getElementById("WD1");
    let velocityFactor;
    let wireType;
    
    if (w1.checked) {
        velocityFactor = 0.9167; // VF for WD-1 wire type
        wireType = 'WD-1';
    } else if (w16.checked) {
        velocityFactor = 0.95; // VF for WF-16 wire type
        wireType = 'WF-16';
    } else {
        alert("Please select a wire type");
        return;
    }
    
    // Scientific constant: 983.571 ft/MHz - speed of light in feet divided by 10^6
    const SCIENTIFIC_CONSTANT = 983.571;
    
    // Get frequency input from the single input field
    const channelInput = document.getElementById("channelInput").value.trim();
    if (!channelInput) {
        alert("Please enter at least one channel frequency");
        return;
    }
    
    // Parse the input into an array of frequencies
    const channels = channelInput.split(",")
        .map(item => {
            return {
                frequency: parseFloat(item.trim()),
                label: null,
                colorIndex: null
            };
        })
        .filter(channel => !isNaN(channel.frequency) && channel.frequency > 0);
    
    if (channels.length === 0) {
        alert("Please enter valid channel frequencies");
        return;
    }
    
    // Calculate wavelengths for each channel using the proper scientific formula
    const channelsWithWavelengths = channels.map(channel => {
        // Calculate full wave using scientific formula: (983.571 × VF) / f_MHz
        const fullWaveValue = (SCIENTIFIC_CONSTANT * velocityFactor) / channel.frequency;
        const antFTfull = Math.floor(fullWaveValue);
        const antINfull = Math.round((fullWaveValue - antFTfull) * 12);
        const fullWaveTX = `${antFTfull} FT ${antINfull} IN`;
        const fullWaveTotalInches = Math.round((antFTfull * 12) + antINfull); // Total inches for comparison
        
        // Calculate half wave values
        const halfWaveValue = fullWaveValue / 2;
        const halfFT = Math.floor(halfWaveValue);
        const halfIN = Math.round((halfWaveValue - halfFT) * 12);
        const halfWaveTX = `${halfFT} FT ${halfIN} IN`;
        const halfWaveTotalInches = Math.round((halfFT * 12) + halfIN); // Total inches for comparison
        
        // Calculate quarter wave values
        const quarterWaveValue = fullWaveValue / 4;
        const quarterFT = Math.floor(quarterWaveValue);
        const quarterIN = Math.round((quarterWaveValue - quarterFT) * 12);
        const quarterWaveTX = `${quarterFT} FT ${quarterIN} IN`;
        const quarterWaveTotalInches = Math.round((quarterFT * 12) + quarterIN); // Total inches for comparison
        
        // Add NVIS capability flag
        const isNVISCapable = channel.frequency >= 1.8 && channel.frequency <= 8.0;
        
        // Determine if good for day or night operation
        const isDayFrequency = channel.frequency > 10.0; // Higher frequencies better during day
        const isNightFrequency = channel.frequency < 7.0; // Lower frequencies better at night
        
        return {
            ...channel,
            fullWave: {
                value: fullWaveValue,
                text: fullWaveTX,
                feet: antFTfull,
                inches: antINfull,
                totalInches: fullWaveTotalInches
            },
            halfWave: {
                value: halfWaveValue,
                text: halfWaveTX,
                feet: halfFT,
                inches: halfIN,
                totalInches: halfWaveTotalInches
            },
            quarterWave: {
                value: quarterWaveValue,
                text: quarterWaveTX,
                feet: quarterFT,
                inches: quarterIN,
                totalInches: quarterWaveTotalInches
            },
            isNVISCapable: isNVISCapable,
            isDayFrequency: isDayFrequency,
            isNightFrequency: isNightFrequency
        };
    });
    
    // Group channels with similar wavelengths (within 5% of each other)
    const wavelengthGroups = groupSimilarWavelengths(channelsWithWavelengths);
    
    // Store the results for later export
    lastCalculationResults = {
        channels: channelsWithWavelengths,
        groups: wavelengthGroups,
        wireType: wireType,
        velocityFactor: velocityFactor
    };
    
    // Render the results
    renderResults(channelsWithWavelengths, wavelengthGroups);
    
    // Show results area
    document.getElementById("resultsArea").style.display = "block";
    
    // Show antenna rankings content by default
    const contentSection = document.getElementById('antennaRankingsContent');
    if (contentSection) {
        contentSection.style.display = 'block';
    }
}

/**
 * Calculate the ranking score for an antenna based on the new criteria
 * 
 * @param {Object} group - The antenna group to score
 * @returns {number} - The calculated score
 */
function calculateAntennaRankingScore(group) {
    let score = 0;
    
    // 1. Primary: Antenna Type Preference
    switch(group.waveType) {
        case 'fullWave':
            score += 100; // Full Wave: highest priority
            break;
        case 'halfWave':
            score += 60;  // Half Wave: medium priority
            break;
        case 'quarterWave':
            score += 20;  // Quarter Wave: lowest priority
            break;
    }
    
    // 2. Day/Night Coverage (Secondary)
    // Check if the antenna group supports both day and night frequencies
    const hasDayFrequency = group.channels.some(channel => channel.isDayFrequency);
    const hasNightFrequency = group.channels.some(channel => channel.isNightFrequency);
    
    // Bonus if the antenna supports both day AND night frequencies
    if (hasDayFrequency && hasNightFrequency) {
        score += 20; // Bonus for supporting both day and night coverage
    }
    
    // 3. Frequency Tie-Breaker will be handled in the sorting function
    
    // Store the score in the group object (not displayed to user)
    group.rankingScore = score;
    group.hasDayNightCoverage = hasDayFrequency && hasNightFrequency;
    
    return score;
}

/**
 * Group channels with similar wavelengths (within 5% of each other)
 * UPDATED: Now includes cross-type wavelength comparison and new ranking system
 */
function groupSimilarWavelengths(channels) {
    // Create unified wavelength groups
    const wavelengthGroups = [];
    let nextColorIndex = 0;
    
    // Create a flat list of all wavelength measurements across all channels
    const allWavelengths = [];
    
    channels.forEach((channel, channelIndex) => {
        // Add each wavelength type as a separate entry
        allWavelengths.push({
            channelIndex: channelIndex,
            frequency: channel.frequency,
            waveType: 'fullWave',
            totalInches: channel.fullWave.totalInches,
            dimensions: channel.fullWave,
            isNVISCapable: channel.isNVISCapable,
            isDayFrequency: channel.isDayFrequency,
            isNightFrequency: channel.isNightFrequency
        });
        
        allWavelengths.push({
            channelIndex: channelIndex,
            frequency: channel.frequency,
            waveType: 'halfWave',
            totalInches: channel.halfWave.totalInches,
            dimensions: channel.halfWave,
            isNVISCapable: channel.isNVISCapable,
            isDayFrequency: channel.isDayFrequency,
            isNightFrequency: channel.isNightFrequency
        });
        
        allWavelengths.push({
            channelIndex: channelIndex,
            frequency: channel.frequency,
            waveType: 'quarterWave',
            totalInches: channel.quarterWave.totalInches,
            dimensions: channel.quarterWave,
            isNVISCapable: channel.isNVISCapable,
            isDayFrequency: channel.isDayFrequency,
            isNightFrequency: channel.isNightFrequency
        });
    });
    
    // Track which wavelength entries have been assigned to groups
    const assignedEntries = new Set();
    
    // For each wavelength entry
    for (let i = 0; i < allWavelengths.length; i++) {
        // Skip if this entry is already assigned
        if (assignedEntries.has(i)) continue;
        
        const currentEntry = allWavelengths[i];
        const entriesInGroup = [i]; // Start with the current entry
        
        // Find all other entries with similar wavelengths
        for (let j = 0; j < allWavelengths.length; j++) {
            if (i !== j && !assignedEntries.has(j)) {
                const compareEntry = allWavelengths[j];
                
                // Use percentage-based comparison
                if (isWithinPercentageTolerance(currentEntry.totalInches, compareEntry.totalInches, 5)) {
                    // Add to group if frequencies are different
                    if (currentEntry.frequency !== compareEntry.frequency) {
                        entriesInGroup.push(j);
                        assignedEntries.add(j);
                    }
                }
            }
        }
        
        // Only create a group if there are multiple channels (multiple frequencies)
        if (entriesInGroup.length > 1) {
            // Mark the seed entry as assigned
            assignedEntries.add(i);
            
            // Count unique frequencies in this group
            const uniqueFrequencies = new Set(
                entriesInGroup.map(index => allWavelengths[index].frequency)
            );
            
            // Only add the group if it supports multiple frequencies
            if (uniqueFrequencies.size > 1) {
                // Get all channels that can use this antenna
                const channelsInGroup = entriesInGroup.map(index => {
                    const entry = allWavelengths[index];
                    return {
                        ...channels[entry.channelIndex],
                        matchedWaveType: entry.waveType
                    };
                });
                
                // Create type labels for the group description
                const typeLabels = {
                    'fullWave': 'Full Wave',
                    'halfWave': 'Half Wave',
                    'quarterWave': 'Quarter Wave'
                };
                
                // Create a new group
                const newGroup = {
                    id: `group-${wavelengthGroups.length}`,
                    type: typeLabels[currentEntry.waveType],
                    waveType: currentEntry.waveType,
                    channels: channelsInGroup,
                    colorIndex: nextColorIndex % colorPalette.length,
                    dimensions: currentEntry.dimensions,
                    entriesInGroup: entriesInGroup.map(idx => allWavelengths[idx]), // Keep all entries for debugging
                    sharedWavelength: currentEntry.totalInches
                };
                
                // Calculate the ranking score for this group
                calculateAntennaRankingScore(newGroup);
                
                wavelengthGroups.push(newGroup);
                nextColorIndex++;
            }
        }
    }
    
    // Second pass: Create additional groups for same wavelength type
    const waveTypes = ['fullWave', 'halfWave', 'quarterWave'];
    const typeLabels = {
        'fullWave': 'Full Wave',
        'halfWave': 'Half Wave',
        'quarterWave': 'Quarter Wave'
    };
    
    // For each wave type
    for (const waveType of waveTypes) {
        // Track which channels have been assigned to groups for this wave type
        const assignedChannels = new Set();
        
        // For each channel
        for (let i = 0; i < channels.length; i++) {
            // Skip if this channel is already assigned for this wave type
            if (assignedChannels.has(i)) continue;
            
            const currentChannel = channels[i];
            const currentInches = currentChannel[waveType].totalInches;
            const channelsInGroup = [i]; // Start with the current channel
            
            // Find other channels with similar wavelengths of the same type
            for (let j = i + 1; j < channels.length; j++) {
                if (!assignedChannels.has(j)) {
                    const compareChannel = channels[j];
                    const compareInches = compareChannel[waveType].totalInches;
                    
                    // Use percentage-based comparison
                    if (isWithinPercentageTolerance(currentInches, compareInches, 5)) {
                        channelsInGroup.push(j);
                        assignedChannels.add(j);
                    }
                }
            }
            
            // Only create a group if there are multiple channels
            if (channelsInGroup.length > 1) {
                assignedChannels.add(i); // Mark the seed channel as assigned
                
                // Check if this group would be a duplicate of an existing group
                let isDuplicate = false;
                for (const group of wavelengthGroups) {
                    // If this is the same wave type and has the same channels
                    if (group.waveType === waveType) {
                        // Get just the frequencies for comparison
                        const groupFreqs = group.channels.map(ch => ch.frequency).sort();
                        const newFreqs = channelsInGroup.map(idx => channels[idx].frequency).sort();
                        
                        // Check if the frequency sets are the same
                        if (groupFreqs.length === newFreqs.length && 
                            groupFreqs.every((f, idx) => Math.abs(f - newFreqs[idx]) < 0.001)) {
                            isDuplicate = true;
                            break;
                        }
                    }
                }
                
                // Only add the group if it's not a duplicate
                if (!isDuplicate) {
                    // Create a new group
                    const newGroup = {
                        id: `${waveType}-${wavelengthGroups.length}`,
                        type: typeLabels[waveType],
                        waveType: waveType,
                        channels: channelsInGroup.map(idx => ({...channels[idx]})),
                        colorIndex: nextColorIndex % colorPalette.length,
                        dimensions: channels[i][waveType]
                    };
                    
                    // Calculate the ranking score for this group
                    calculateAntennaRankingScore(newGroup);
                    
                    wavelengthGroups.push(newGroup);
                    nextColorIndex++;
                }
            }
        }
    }
    
    // Sort groups based on the new ranking system
    wavelengthGroups.sort((a, b) => {
        // First, sort by ranking score
        const scoreDiff = b.rankingScore - a.rankingScore;
        if (scoreDiff !== 0) return scoreDiff;
        
        // If equal score, use frequency as tie-breaker (prioritize lowest frequency for distance)
        // This implements the "longer distances" tie-breaker by choosing the lowest frequency
        const aLowestFreq = Math.min(...a.channels.map(ch => ch.frequency));
        const bLowestFreq = Math.min(...b.channels.map(ch => ch.frequency));
        return aLowestFreq - bLowestFreq; // Lower frequency first (better for distance)
        
        // Note: To implement "faster data" tie-breaker, you would use:
        // return bHighestFreq - aHighestFreq; // Higher frequency first (better for data)
    });
    
    return wavelengthGroups;
}

/**
 * Check if two values are within a percentage tolerance using the specified formula
 * Percentage-based comparison: |W1-W2|/average(W1,W2) × 100 ≤ 5%
 */
function isWithinPercentageTolerance(value1, value2, percentageTolerance) {
    // Avoid division by zero
    if (value1 === 0 && value2 === 0) return true;
    if (value1 === 0 || value2 === 0) return false;
    
    // Calculate the average
    const average = (value1 + value2) / 2;
    
    // Calculate the absolute difference
    const absoluteDifference = Math.abs(value1 - value2);
    
    // Calculate the percentage difference
    const percentageDifference = (absoluteDifference / average) * 100;
    
    // Return true if the percentage difference is less than or equal to the tolerance
    return percentageDifference <= percentageTolerance;
}

/**
 * Render the results to the page
 * UPDATED: Now includes NVIS capability column
 */
function renderResults(channels, wavelengthGroups) {
    // Get all HTML elements needed
    const resultsTable = document.getElementById("resultsTable");
    const rankedAntennas = document.getElementById("rankedAntennas");
    const colorLegend = document.getElementById("colorLegend");
    
    // Clear previous results
    resultsTable.innerHTML = "";
    rankedAntennas.innerHTML = "";
    colorLegend.innerHTML = "";
    
    // Create table header with new NVIS column
    const headerRow = document.createElement("tr");
    headerRow.className = "style2";
    headerRow.innerHTML = `
        <th class="style2">Channel (MHz)</th>
        <th class="style2">Full Wave</th>
        <th class="style2">1/2 Wave</th>
        <th class="style2">1/4 Wave</th>
        <th class="style2">NVIS Capable</th>
    `;
    resultsTable.appendChild(headerRow);
    
    // Create a mapping of channel-wavelength combinations to their color assignments
    const colorMapping = {};
    
    // Map wave types to their property names in the wavelength groups
    const waveTypeToProperty = {
        'fullWave': 'fullWave',
        'halfWave': 'halfWave',
        'quarterWave': 'quarterWave'
    };
    
    // Initialize color mapping for each channel and wavelength type
    wavelengthGroups.forEach(group => {
        const color = colorPalette[group.colorIndex];
        
        group.channels.forEach(channel => {
            const key = `${channel.frequency}-${channel.matchedWaveType || group.waveType}`;
            if (!colorMapping[key]) {
                colorMapping[key] = color;
            }
        });
    });
    
    // Sort channels by frequency (lowest to highest)
    const sortedChannels = [...channels].sort((a, b) => a.frequency - b.frequency);
    
    // Add rows for each channel
    sortedChannels.forEach(channel => {
        const row = document.createElement("tr");
        row.className = "style2";
        
        // Get the colors for each wavelength type
        const fullWaveColor = colorMapping[`${channel.frequency}-fullWave`] || 'transparent';
        const halfWaveColor = colorMapping[`${channel.frequency}-halfWave`] || 'transparent';
        const quarterWaveColor = colorMapping[`${channel.frequency}-quarterWave`] || 'transparent';
        
        // Check if frequency is NVIS-capable (1.8-8.0 MHz)
        const isNVISCapable = channel.frequency >= 1.8 && channel.frequency <= 8.0;
        
        row.innerHTML = `
            <td class="style2">${channel.frequency.toFixed(3)}</td>
            <td class="style2" style="background-color: ${fullWaveColor}">${channel.fullWave.text}</td>
            <td class="style2" style="background-color: ${halfWaveColor}">${channel.halfWave.text}</td>
            <td class="style2" style="background-color: ${quarterWaveColor}">${channel.quarterWave.text}</td>
            <td class="style2">${isNVISCapable ? 'Yes' : 'No'}</td>
        `;
        
        resultsTable.appendChild(row);
    });
    
    // Render the antenna rankings with the same colors
    renderAntennaRankings(wavelengthGroups);
    
    // Render color legend with the same colors
    renderColorLegend(wavelengthGroups);
}

/**
 * Render the antenna rankings section
 * UPDATED: Now includes NVIS capability and day/night coverage info
 */
function renderAntennaRankings(wavelengthGroups) {
    const rankedAntennasDiv = document.getElementById("rankedAntennas");
    
    // If no multi-channel antennas found, display a message
    if (wavelengthGroups.length === 0) {
        rankedAntennasDiv.innerHTML = `
            <div class="no-rankings">
                <p>No multi-channel antennas found. Try entering frequencies that are closer together.</p>
                <p>Hint: Frequencies with wavelengths within 5% of each other can share the same antenna.</p>
            </div>
        `;
        return;
    }
    
    // Create a ranking card for each antenna group
    wavelengthGroups.forEach((group, index) => {
        const antennaCard = document.createElement("div");
        antennaCard.className = "antenna-card";
        const bgColor = colorPalette[group.colorIndex];
        
        // Use a lighter version of the color for the card border
        antennaCard.style.borderColor = bgColor;
        
        // Create card header with rank and antenna type
        const antennaHeader = document.createElement("div");
        antennaHeader.className = "antenna-header";
        
        // Create rank badge with permanent rank number
        const rankBadge = document.createElement("div");
        rankBadge.className = "rank-badge";
        rankBadge.style.backgroundColor = bgColor;
        rankBadge.textContent = `#${index + 1}`;
        
        // Create antenna title with dimensions
        const antennaTitle = document.createElement("div");
        antennaTitle.className = "antenna-title";
        antennaTitle.innerHTML = `
            <span class="antenna-type">${group.type}</span>: 
            <span class="antenna-dimensions">${group.dimensions.text}</span>
        `;
        
        // Create coverage info with channel count
        const coverageInfo = document.createElement("div");
        coverageInfo.className = "coverage-info";
        coverageInfo.innerHTML = `
            <span class="channel-count">${group.channels.length}</span> 
            channel${group.channels.length > 1 ? 's' : ''}
        `;
        
        // Assemble the header
        antennaHeader.appendChild(rankBadge);
        antennaHeader.appendChild(antennaTitle);
        antennaHeader.appendChild(coverageInfo);
        
        // Create details section
        const antennaDetails = document.createElement("div");
        antennaDetails.className = "antenna-details";
        
        // Get the wire type and velocity factor
        const wireType = document.getElementById("WF16").checked ? 'WF-16' : 'WD-1';
        const velocityFactor = document.getElementById("WF16").checked ? 0.95 : 0.9167;
        
        // Check for NVIS capability
        const nvisCapableChannels = group.channels.filter(channel => 
            channel.frequency >= 1.8 && channel.frequency <= 8.0
        );
        const hasNVISCapability = nvisCapableChannels.length > 0;
        
        // Add antenna specifications - now including NVIS capability and day/night coverage
        const antennaSpecs = document.createElement("div");
        antennaSpecs.className = "antenna-specs";
        antennaSpecs.innerHTML = `
            <div class="spec-item">
                <span class="spec-label">Wire Type:</span> 
                <span class="spec-value">${wireType}</span>
            </div>
            <div class="spec-item">
                <span class="spec-label">Velocity Factor:</span> 
                <span class="spec-value">${velocityFactor}</span>
            </div>
            <div class="spec-item">
                <span class="spec-label">Length:</span> 
                <span class="spec-value">${group.dimensions.feet} ft ${group.dimensions.inches} in</span>
            </div>
            <div class="spec-item">
                <span class="spec-label">NVIS Capable:</span> 
                <span class="spec-value">${hasNVISCapability ? 'Yes' : 'No'}</span>
            </div>
            <div class="spec-item">
                <span class="spec-label">Day/Night Coverage:</span> 
                <span class="spec-value">${group.hasDayNightCoverage ? 'Good' : 'Limited'}</span>
            </div>
        `;
        
        // Add supported channels list
        const supportedChannels = document.createElement("div");
        supportedChannels.className = "supported-channels";
        supportedChannels.innerHTML = "<strong>Supported Channels:</strong> ";
        
        // Add a pill for each supported channel
        const uniqueFrequencies = new Set();
        group.channels.forEach(channel => {
            // Only add each frequency once
            if (!uniqueFrequencies.has(channel.frequency)) {
                uniqueFrequencies.add(channel.frequency);
                
                const channelPill = document.createElement("span");
                channelPill.className = "channel-pill";
                channelPill.textContent = `${channel.frequency.toFixed(3)} MHz`;
                channelPill.style.backgroundColor = bgColor;
                
                // Add NVIS indicator if applicable
                if (channel.frequency >= 1.8 && channel.frequency <= 8.0) {
                    channelPill.textContent += ' (NVIS)';
                }
                
                // Add Day/Night indicator 
                if (channel.isDayFrequency) {
                    channelPill.textContent += ' (Day)';
                } else if (channel.isNightFrequency) {
                    channelPill.textContent += ' (Night)';
                }
                
                supportedChannels.appendChild(channelPill);
            }
        });
        
        // Assemble the details section
        antennaDetails.appendChild(antennaSpecs);
        antennaDetails.appendChild(supportedChannels);
        
        // Assemble the complete card
        antennaCard.appendChild(antennaHeader);
        antennaCard.appendChild(antennaDetails);
        
        // Add the card to the rankings area
        rankedAntennasDiv.appendChild(antennaCard);
    });
}

/**
 * Render the color legend
 */
function renderColorLegend(wavelengthGroups) {
    const colorLegendDiv = document.getElementById("colorLegend");
    
    // If no groups, don't show a legend
    if (wavelengthGroups.length === 0) {
        colorLegendDiv.style.display = "none";
        return;
    }
    
    colorLegendDiv.style.display = "block";
    
    // Create legend title
    const legendTitle = document.createElement("div");
    legendTitle.className = "legend-title";
    legendTitle.innerHTML = "<strong>Wavelength Grouping Legend</strong>";
    colorLegendDiv.appendChild(legendTitle);
    
    // Create legend description
    const legendDesc = document.createElement("div");
    legendDesc.className = "legend-description";
    legendDesc.textContent = "Channels with wavelengths within 5% tolerance share the same color and can use the same antenna";
    colorLegendDiv.appendChild(legendDesc);
    
    // Create container for legend items
    const legendItems = document.createElement("div");
    legendItems.className = "legend-items";
    colorLegendDiv.appendChild(legendItems);
    
    // Add a legend item for each group
    wavelengthGroups.forEach((group, index) => {
        const legendItem = document.createElement("div");
        legendItem.className = "legend-item";
        
        // Create color box
        const colorBox = document.createElement("div");
        colorBox.className = "legend-color";
        colorBox.style.backgroundColor = colorPalette[group.colorIndex];
        
        // Create text with antenna type and dimensions
        const legendText = document.createElement("span");
        legendText.className = "legend-text";
        
        // Get unique frequencies for this group
        const uniqueFrequencies = Array.from(new Set(
            group.channels.map(c => c.frequency)
        )).sort((a, b) => a - b);
        
        // Format frequencies list with NVIS indicators
        const channelFrequencies = uniqueFrequencies
            .map(f => {
                const isNVIS = f >= 1.8 && f <= 8.0;
                const isDayFreq = f > 10.0;
                const isNightFreq = f < 7.0;
                
                let label = `${f.toFixed(3)}`;
                if (isNVIS) label += ' (NVIS)';
                if (isDayFreq) label += ' (Day)';
                else if (isNightFreq) label += ' (Night)';
                
                return label;
            })
            .join(", ");
        
        legendText.innerHTML = `
            <strong>#${index + 1}:</strong> ${group.type} ${group.dimensions.text} (${group.dimensions.totalInches} in)
            <br><span class="legend-channels">${channelFrequencies} MHz</span>
        `;
        
        // Assemble the legend item
        legendItem.appendChild(colorBox);
        legendItem.appendChild(legendText);
        
        // Add to legend items container
        legendItems.appendChild(legendItem);
    });
}

/**
 * Print results directly
 */
function printResults() {
    if (!lastCalculationResults.channels || lastCalculationResults.channels.length === 0) {
        alert("No calculation results to print. Please calculate first.");
        return;
    }
    
    window.print();
}

/**
 * Clear the input and results
 */
function clearInput() {
    // Clear the text input field
    document.getElementById("channelInput").value = "";
    
    // Hide the results area
    document.getElementById("resultsArea").style.display = "none";
    
    // Reset the wire type radio buttons
    document.getElementById("WF16").checked = false;
    document.getElementById("WD1").checked = false;
    
    // Clear stored results
    lastCalculationResults = {
        channels: [],
        groups: []
    };
}